#!/usr/bin/env python3

from greenstalk import Client

c = Client(("localhost", 11300))

print("beanstalkd-stats:")
print(c.stats())
while True:
    print("Reserving message")
    job = c.reserve()
    print("new message:")
    print(job.body)
    c.delete(job)
c.close()



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

supported_s3_events = {
    "*",
    "s3:TestEvent",
    # object
    "s3:Object*",
    "s3:ObjectCreated:*",
    "s3:ObjectCreated:Put",
    "s3:ObjectCreated:Post",
    "s3:ObjectCreated:Copy",
    "s3:ObjectRemoved:*",
    "s3:ObjectRemoved:Delete",
    "s3:ObjectAccessed:*",
    "s3:ObjectAccessed:Get",
    "s3:ObjectAccessed:Head",
    # bucket
    "s3:Bucket*",
    "s3:BucketCreated:*",
    "s3:BucketCreated:Put",
    "s3:BucketCreated:Post",
    "s3:BucketCreated:Copy",
    "s3:BucketRemoved:*",
    "s3:BucketRemoved:Delete",
    "s3:BucketAccessed:*",
    "s3:BucketAccessed:Get",
    "s3:BucketAccessed:Head",
    # account
    "s3:Account*",
    "s3:AccountCreated:*",
    "s3:AccountCreated:Put",
    "s3:AccountCreated:Post",
    "s3:AccountCreated:Copy",
    "s3:AccountRemoved:*",
    "s3:AccountRemoved:Delete",
    "s3:AccountAccessed:*",
    "s3:AccountAccessed:Head",
    "s3:AccountAccessed:Get",
}



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enoss.filter_rules.irule import IRule
from swift.proxy.controllers.base import get_object_info


class ContenttypeRule(IRule):

    @staticmethod
    def validate(value):
        return type(value) == str

    def __call__(self, app, resp):
        method = resp.environ.get('swift.orig_req_method', resp.request.method)
        content_type = None
        if method == "PUT":
            # read content type from request headers
            content_type = resp.environ.get("CONTENT_TYPE")
        elif method in ["GET", "HEAD"]:
            # read content type from respond headers
            content_type = resp.headers.get("Content-Type")
        else:
            # read content type from object storage
            object_info = get_object_info(resp.environ, app)
            if object_info["status"] == 200:
                # if object exists in storage
                content_type = object_info.get("type")
        return content_type and content_type == self.value



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
import six


@six.add_metaclass(abc.ABCMeta)
class IDestination(object):
    @abc.abstractmethod
    def __init__(self, conf):
        raise NotImplementedError('__init__ is not implemented')

    @abc.abstractmethod
    def send_notification(self, notification):
        raise NotImplementedError('send_notification is not implemented')



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

from swift.common.swob import wsgify, HTTPForbidden, HTTPBadRequest, \
    HTTPServerError
from swift.common.utils import split_path, get_logger
from swift.common.request_helpers import get_sys_meta_prefix
from swift.proxy.controllers.base import get_container_info, get_account_info,\
    get_object_info, get_info
from swift.common.wsgi import WSGIContext

import enoss.destinations as destinations_module
import enoss.payloads as payloads_module

from enoss.configuration import (
    ConfigurationInvalid, S3ConfigurationValidator, S3NotifiationConfiguration)
from enoss.utils import (
    get_payload_handlers, get_destination_handlers, get_payload_handler_name,
    get_destination_handler_name, json_object_hook)
import json
import os
from six.moves.configparser import ConfigParser


class ENOSSMiddleware(WSGIContext):
    def __init__(self, app, conf, logger=None):
        self.app = app
        self.conf = conf
        self.logger = logger or get_logger(conf,
                                           log_route='eventnotifications')
        self.configuration_validator = S3ConfigurationValidator(
            self.conf["s3_schema"])
        self._load_destinations_conf()
        self._load_destination_handlers()
        self._load_payload_handlers()
        self._load_admin_s3_conf()
        super(ENOSSMiddleware, self).__init__(app)

    def _load_destinations_conf(self):
        cfg_parser = ConfigParser()
        if cfg_parser.read(self.conf["destinations_conf_path"]):
            self.destinations_conf = cfg_parser._sections
        else:
            raise Exception("Cannot load destinations configuration {}".format(
                self.conf["destinations_conf_path"]))

    def _load_destination_handlers(self):
        self.destination_handlers = {}
        dest_handlers = get_destination_handlers([destinations_module])
        use_dests = []
        for dest in self.conf["use_destinations"].split(","):
            dest = dest.strip()
            if dest:
                use_dests.append(dest)
        self.logger.info("avaliable enoss destinations:{}".format(
            ",".join(use_dests))
        )
        for dest_name in use_dests:
            handler_name = get_destination_handler_name(dest_name)
            handler = dest_handlers[handler_name]
            self.destination_handlers[handler_name] = handler(
                self.destinations_conf
            )

    def _load_payload_handlers(self):
        payload_handlers = get_payload_handlers([payloads_module])
        self.payload_handlers = {handler_name: payload_handler(self.conf)
                                 for handler_name, payload_handler
                                 in payload_handlers.items()}

    def _load_admin_s3_conf(self):
        self.admin_s3_conf = None
        admin_s3_conf_path = self.conf.get("admin_s3_conf_path")
        if admin_s3_conf_path and os.path.isfile(admin_s3_conf_path):
            try:
                with open(admin_s3_conf_path) as f:
                    admin_s3_conf = json.loads(
                        f.read(),
                        object_hook=json_object_hook
                    )
                    self.configuration_validator.validate(
                        self.destination_handlers,
                        self.payload_handlers,
                        admin_s3_conf)
                    self.admin_s3_conf = admin_s3_conf
            except Exception as e:
                self.logger.error("error during loading admin s3 conf:{}".
                                  format(e))

    def get_notification_configuration(self, info_method, environ):
        info = info_method(environ, self.app)
        notifications_conf = info.get("sysmeta", {}).get("notifications")
        return notifications_conf

    def get_current_level(self, account, container, object):
        if object:
            return "object"
        elif container:
            return "container"
        elif account:
            return "account"
        else:
            return None

    def _get_upper_level_confs(self, curr_level, req):
        confs = [self.admin_s3_conf] if self.admin_s3_conf else []
        if curr_level in ["object", "container"]:
            notifications_conf = self.get_notification_configuration(
                get_account_info, req.environ)
            if notifications_conf:
                confs.append(notifications_conf)
        if curr_level == "object":
            notifications_conf = self.get_notification_configuration(
                get_container_info, req.environ)
            if notifications_conf:
                confs.append(notifications_conf)
        return confs

    def _read_info_before_delete(self, req):
        version, account, container, object = split_path(
            req.environ['PATH_INFO'], 1, 4, rest_with_last=True)
        if object:
            get_object_info(req.environ, self.app)
        elif container or account:
            get_info(self.app, req.environ, account, container)

    def send_test_notification(self, curr_level, req):
        # todo check if curr_level is not None
        info_method = get_container_info if curr_level == "container" \
            else get_account_info
        notifications_conf = self.get_notification_configuration(
            info_method, req.environ)
        if notifications_conf:
            s3_conf = S3NotifiationConfiguration(notifications_conf)
            for destination_name, destination_configurations in \
                    s3_conf.destinations_configurations.items():
                handler_name = get_destination_handler_name(destination_name)
                destination_handler = self.destination_handlers[handler_name]
                for destination_configuration in destination_configurations:
                    handler_name = get_payload_handler_name(
                        destination_configuration.payload_type)
                    payload_handler = self.payload_handlers[handler_name]
                    payload = payload_handler.create_test_payload(
                        self.app, req, destination_configuration)
                    destination_handler.send_notification(payload)

    def send_notification(self, upper_level_confs, req):
        for notification_conf in upper_level_confs:
            try:
                # in case some invalid configuration is stored
                s3_conf = S3NotifiationConfiguration(notification_conf)
            except Exception as e:
                self.logger.error("{}".format(e))
                continue
            satisfied_destinations = s3_conf.get_satisfied_destinations(
                self.app, req)
            for destination_name, destination_configurations in \
                    satisfied_destinations.items():
                handler_name = get_destination_handler_name(destination_name)
                destination_handler = self.destination_handlers[handler_name]
                for destination_configuration in destination_configurations:
                    handler_name = get_payload_handler_name(
                        destination_configuration.payload_type)
                    payload_handler = self.payload_handlers[handler_name]
                    payload = payload_handler.create_payload(
                        self.app, req, destination_configuration)
                    destination_handler.send_notification(payload)

    def _post_notification(self, curr_level, req):
        if curr_level not in ["account", "container"]:
            # deny notification set on object level
            return HTTPForbidden(request=req)

        notification_sysmeta = get_sys_meta_prefix(curr_level) \
            + 'notifications'
        # todo can contain too many white spaces in body
        # maybe transform to json then to str back to remove them
        config = req.body.decode()
        if not config:
            # body is empty => delete stored configuration
            req.headers[notification_sysmeta] = ""
            return
        try:
            config_json = json.loads(config, object_hook=json_object_hook)
            self.configuration_validator.validate(
                self.destination_handlers,
                self.payload_handlers,
                config_json)
            # eliminate whitespaces
            config = json.dumps(config_json, separators=(',', ':'))
            req.headers[notification_sysmeta] = config
        except ConfigurationInvalid as e:
            return HTTPBadRequest(request=req, content_type='text/plain',
                                  body=str(e))
        except json.JSONDecodeError:
            return HTTPBadRequest(request=req, content_type='text/plain',
                                  body="Invalid json")
        except Exception as e:
            self.logger.error("error during posting notification "
                "configuration to sysmeta: {}".format(e))
            return HTTPServerError(request=req)

    def _get_notification(self, curr_level, resp):
        info_method = get_container_info if curr_level == "container" \
            else get_account_info
        conf = self.get_notification_configuration(
            info_method, resp.environ)
        resp.body = str.encode(conf if conf else '')

    @wsgify
    def __call__(self, req):
        if req.headers.get("X-Backend-EventNotification-Ignore"):
            return req.get_response(self.app)
        # swift can call it self recursively
        # => we want only one notification per user request
        req.headers["X-Backend-EventNotification-Ignore"] = True

        version, account, container, object = split_path(
            req.environ['PATH_INFO'], 1, 4, rest_with_last=True)
        curr_level = self.get_current_level(account, container, object)
        event_configation_changed = False
        if req.method == "POST" and req.query_string == "notification":
            resp_err = self._post_notification(curr_level, req)
            if not resp_err:
                event_configation_changed = True
            else:
                # forbidden, bad request or server error
                return resp_err

        if req.method == "DELETE":
            self._read_info_before_delete(req)

        # get swift response
        resp = req.get_response(self.app)

        upper_level_confs = self._get_upper_level_confs(curr_level, resp)
        try:
            # sending notifications can be unsuccessful and throw exceptions
            if event_configation_changed:
                self.send_test_notification(curr_level, resp)
            self.send_notification(upper_level_confs, resp)
        except Exception as e:
            self.logger.error("error:{}".format(e))
        # todo: better way to test query_string
        if (req.method == "GET" and req.query_string
                and req.query_string.startswith("notification")
                and resp.is_success):  # todo ACL
            self._get_notification(curr_level, resp)
        return resp


def enoss_factory(global_conf, **local_conf):
    conf = global_conf.copy()
    conf.update(local_conf)

    def enoss_factory(app):
        return ENOSSMiddleware(app, conf)
    return enoss_factory



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
from email.utils import parsedate
import time

from enoss.payloads.ipayload import IPayload
from enoss.utils import get_s3_event_name
from swift.common.utils import split_path
from swift.proxy.controllers.base import get_object_info


def _get_object_info(object, response, app):
    obj_info = {}
    method = response.environ.get(
            'swift.orig_req_method', response.request.method)
    if method in ["GET", "HEAD"]:
        obj_info["eTag"] =  response.headers["Etag"]
        obj_info["length"] = response.headers["Content-Length"]
    elif method == "PUT":
        obj_info["eTag"] = response.headers["Etag"]
        obj_info["length"] = response.request.headers["Content-Length"]
    else:
        obj_info = get_object_info(response.environ, app)
    return obj_info


class S3Payload(IPayload):

    def create_test_payload(self, app, request, invoking_configuration):
        version, account, container, object = split_path(
            request.environ['PATH_INFO'], 1, 4, rest_with_last=True)
        container = container if isinstance(container, str) else ''
        account = account if isinstance(account, str) else ''

        event = {
            "Service": "Amazon S3",
            "Event": "s3:TestEvent",
            "Time": datetime.now().isoformat(),
            "Bucket": container,
            "RequestId": request.environ.get("swift.trans_id"),
            "HostId": "TODO"
        }
        if container:
            event["Bucket"] = container
        else:
            event["Account"] = account
        return event

    def create_payload(self, app, request, invoking_configuration):
        version, account, container, object = split_path(
            request.environ['PATH_INFO'], 1, 4, rest_with_last=True)
        object = object if isinstance(object, str) else ''
        container = container if isinstance(container, str) else ''
        account = account if isinstance(account, str) else ''

        obj_info = _get_object_info(object, request, app) \
		   if object else {}

        method = request.environ.get(
            'swift.orig_req_method', request.request.method)
        event_name = get_s3_event_name(account, container, object, method)
        object_vesion_id = request.headers.get("X-Object-Version-Id", '')
        sequencer = ""
        if method in ["PUT", "DELETE"]:
            if object_vesion_id:
                # version_id has more accurate timestamp
                sequencer = object_vesion_id
            elif 'Last-Modified' in request.headers:
                sequencer = time.mktime(
                    parsedate(request.headers['Last-Modified']))
        notification_payload = {
            "Records": [{
                "eventVersion": "2.2",
                "eventSource": "swift:s3",
                # todo: check if req has timestamp
                "eventTime": datetime.now().isoformat(),
                "eventName": event_name,
                "userIdentity": {
                    "principalId": request.environ.get("REMOTE_USER")
                },
                "requestParameters": {
                    "sourceIPAddress": request.environ.get("REMOTE_ADDR")
                },
                "responseElements": {
                    "x-amz-request-id": request.environ.get("swift.trans_id")
                    # todo: x-amz-host-id
                },
                "s3": {
                    "s3SchemaVersion": "1.0",
                    "configurationId": "todo",
                    "bucket": {
                        "name": container,
                        "ownerIdentity": {
                            "principalId": account
                        },
                        "arn": "arn:aws:s3:::" + container,
                    },
                    "object": {
                        "key": object,
                        "size": obj_info.get("length") if obj_info else 0,
                        "eTag": obj_info.get("eTag") if obj_info else 0,
                        "versionId": object_vesion_id if object else '',
                        "sequencer": sequencer
                    }
                }
            }]
        }
        return notification_payload



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enoss.filter_rules.irule import IRule
from swift.common.utils import split_path
from swift.proxy.controllers.base import get_account_info, get_container_info,\
    get_object_info


def _get_size(resp, app):
    size = None
    version, account, container, object = split_path(
        resp.environ['PATH_INFO'], 1, 4, rest_with_last=True)
    if object:
        size = get_object_info(resp.environ, app).get("length")
    else:
        info_method = get_container_info if container else get_account_info
        size = info_method(resp.environ, app).get("bytes")
    return size


class MaxsizeRule(IRule):

    @staticmethod
    def validate(value):
        return type(value) == int

    def __call__(self, app, resp):
        size = _get_size(resp, app)
        return size is not None and self.value >= size


class MinsizeRule(IRule):

    @staticmethod
    def validate(value):
        return type(value) == int

    def __call__(self, app, resp):
        size = _get_size(resp, app)
        return size is not None and self.value <= size



#!/usr/bin/python

# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>
# Copyright (c) 2010-2012 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Inspired/Retrived from test_container.py

import json
import unittest
from uuid import uuid4

from test.functional import check_response, retry, load_constraint, SkipTest, \
    requires_acls
import test.functional as tf

import six
from copy import deepcopy


def setUpModule():
    tf.setup_package()


def tearDownModule():
    tf.teardown_package()


class TestENOSS(unittest.TestCase):

    def setUp(self):
        if tf.skip:
            raise SkipTest
        self.name = uuid4().hex
        # this container isn't created by default, but will be cleaned up
        self.container = uuid4().hex

        def put(url, token, parsed, conn):
            conn.request('PUT', parsed.path + '/' + self.name, '',
                         {'X-Auth-Token': token})
            return check_response(conn)

        resp = retry(put)
        resp.read()
        # If the request was received and processed but the container-server
        # timed out getting the response back to the proxy, or the proxy timed
        # out getting the response back to the client, the next retry will 202
        self.assertIn(resp.status, (201, 202))

        self.max_meta_count = load_constraint('max_meta_count')
        self.max_meta_name_length = load_constraint('max_meta_name_length')
        self.max_meta_overall_size = load_constraint('max_meta_overall_size')
        self.max_meta_value_length = load_constraint('max_meta_value_length')

        self.s3_conf_good = {
            "BeanstalkdConfigrations": [
                {
                    "Id": "test",
                    "Events": ["*"],
                    "PayloadStructure": "s3",
                    "Filter": {
                        "Key": {
                            "FilterRules": [{
                                "Name": "suffix",
                                "Value": ".jpg"
                            }]
                        }
                    }
                }
            ]
        }

    def tearDown(self):
        if tf.skip:
            raise SkipTest

        def get(url, token, parsed, conn, container):
            conn.request(
                'GET', parsed.path + '/' + container + '?format=json', '',
                {'X-Auth-Token': token})
            return check_response(conn)

        def delete(url, token, parsed, conn, container, obj):
            if six.PY2:
                obj_name = obj['name'].encode('utf8')
            else:
                obj_name = obj['name']
            path = '/'.join([parsed.path, container, obj_name])
            conn.request('DELETE', path, '', {'X-Auth-Token': token})
            return check_response(conn)

        for container in (self.name, self.container):
            while True:
                resp = retry(get, container)
                body = resp.read()
                if resp.status == 404:
                    break
                self.assertEqual(resp.status // 100, 2, resp.status)
                objs = json.loads(body)
                if not objs:
                    break
                for obj in objs:
                    resp = retry(delete, container, obj)
                    resp.read()
                    # Under load, container listing may not upate immediately,
                    # so we may attempt to delete the same object multiple
                    # times. Tolerate the object having already been deleted.
                    self.assertIn(resp.status, (204, 404))

        def delete(url, token, parsed, conn, container):
            conn.request('DELETE', parsed.path + '/' + container, '',
                         {'X-Auth-Token': token})
            return check_response(conn)

        for container in (self.name, self.container):
            resp = retry(delete, container)
            resp.read()
            # self.container may not have been created at all, but even if it
            # has, for either container there may be a failure that trips the
            # retry despite the request having been successfully processed.
            self.assertIn(resp.status, (204, 404))

    def post_account(self, url, token, parsed, conn, headers):
        new_headers = dict({'X-Auth-Token': token}, **headers)
        conn.request('POST', parsed.path, '', new_headers)
        return check_response(conn)

    def post_notification(self, url, token, parsed, conn, body=''):
        path = parsed.path + '/' + self.name + "?notification"
        headers = {'X-Auth-Token': token,
                   'Content-Type': 'application/json'}
        body = json.dumps(body) if body and isinstance(body, dict) else body
        conn.request('POST', path, body, headers)
        return check_response(conn)

    def get_notification(self, url, token, parsed, conn):
        path = parsed.path + '/' + self.name + "?notification"
        headers = {'X-Auth-Token': token}
        conn.request('GET', path, '', headers)
        return check_response(conn)

    def initial_state(self):
        resp = retry(self.post_notification)
        self.assertIn(resp.status, (200, 204))
        resp = retry(self.get_notification)
        self.assertIn(resp.status, (200, 204))
        self.assertFalse(resp.text)

    def test_POST_GET_GOOD_config(self):
        if tf.skip:
            raise SkipTest
        pass

        # store notification config
        resp = retry(self.post_notification, self.s3_conf_good)
        self.assertIn(resp.status, (200, 204))
        resp = retry(self.get_notification)
        self.assertIn(resp.status, (200, 204))
        self.assertTrue(self.s3_conf_good.items() <= resp.json().items())

        # delete notification config
        resp = retry(self.post_notification)
        self.assertIn(resp.status, (200, 204))
        resp = retry(self.get_notification)
        self.assertIn(resp.status, (200, 204))
        self.assertFalse(resp.text)

    def test_POST_GET_BAD_GOD_config(self):
        if tf.skip:
            raise SkipTest

        # missing mandatory information
        s3_conf_bad = deepcopy(self.s3_conf_good)
        del s3_conf_bad["BeanstalkdConfigrations"][0]["Events"]
        resp = retry(self.post_notification, s3_conf_bad)
        self.assertEqual(resp.status, 400)
        resp = retry(self.get_notification)
        self.assertIn(resp.status, (200, 204))
        self.assertFalse(resp.text)

        # invalid/unsupported event type
        s3_conf_bad["BeanstalkdConfigrations"][0]["Events"] = ["InvalidType"]
        resp = retry(self.post_notification, s3_conf_bad)
        self.assertEqual(resp.status, 400)
        resp = retry(self.get_notification)
        self.assertIn(resp.status, (200, 204))
        self.assertFalse(resp.text)

        # invalid json config
        invalid_json_str = "{test: 1}"
        resp = retry(self.post_notification, invalid_json_str)
        self.assertEqual(resp.status, 400)
        resp = retry(self.get_notification)
        self.assertIn(resp.status, (200, 204))
        self.assertFalse(resp.text)

        # if config is invalid it should not update stored config in swift
        resp = retry(self.post_notification, self.s3_conf_good)
        self.assertIn(resp.status, (200, 204))
        resp = retry(self.get_notification)
        self.assertIn(resp.status, (200, 204))
        self.assertTrue(self.s3_conf_good.items() <= resp.json().items())
        stored_config = resp.json()
        resp = retry(self.post_notification, s3_conf_bad)
        self.assertEqual(resp.status, 400)
        resp = retry(self.get_notification)
        self.assertIn(resp.status, (200, 204))
        self.assertEqual(stored_config, resp.json())

        # return to initial state
        self.initial_state()

    @requires_acls
    def test_acl_get_config(self):
        # store notification config
        resp = retry(self.post_notification, self.s3_conf_good)
        self.assertIn(resp.status, (200, 204))
        resp = retry(self.get_notification)
        self.assertIn(resp.status, (200, 204))
        self.assertTrue(self.s3_conf_good.items() <= resp.json().items())

        # cannot read config
        resp = retry(self.get_notification, use_account=3)
        resp.read()
        self.assertEqual(resp.status, 403)

        # set read acl
        acl_user = tf.swift_test_user[2]
        acl = {'read-only': [acl_user]}
        headers = {'x-account-access-control': json.dumps(acl)}
        resp = retry(self.post_account, headers=headers, use_account=1)
        resp.read()
        self.assertEqual(resp.status, 204)

        # can read config with read-only acl
        resp = retry(self.get_notification, use_account=3)
        self.assertIn(resp.status, (200, 204))
        self.assertTrue(self.s3_conf_good.items() <= resp.json().items())

        # set read-write acl
        acl_user = tf.swift_test_user[2]
        acl = {'read-write': [acl_user]}
        headers = {'x-account-access-control': json.dumps(acl)}
        resp = retry(self.post_account, headers=headers, use_account=1)
        resp.read()
        self.assertEqual(resp.status, 204)

        # can read config with read-write acl
        resp = retry(self.get_notification, use_account=3)
        self.assertIn(resp.status, (200, 204))
        self.assertTrue(self.s3_conf_good.items() <= resp.json().items())

        # delete notification config
        resp = retry(self.post_notification, use_account=3)
        resp.read()
        self.assertIn(resp.status, (200, 204))
        resp = retry(self.get_notification, use_account=3)
        self.assertIn(resp.status, (200, 204))
        self.assertFalse(resp.text)

        # return to initial state
        self.initial_state()

    @requires_acls
    def test_acl_post_config(self):

        # no acl at all
        resp = retry(self.post_notification, self.s3_conf_good, use_account=3)
        resp.read()
        self.assertEqual(resp.status, 403)
        resp = retry(self.get_notification)
        self.assertIn(resp.status, (200, 204))
        self.assertFalse(resp.text)

        # set read acl
        acl_user = tf.swift_test_user[2]
        acl = {'read-only': [acl_user]}
        headers = {'x-account-access-control': json.dumps(acl)}
        resp = retry(self.post_account, headers=headers, use_account=1)
        resp.read()
        self.assertEqual(resp.status, 204)

        # read acl still cannot post notification config
        resp = retry(self.post_notification, self.s3_conf_good, use_account=3)
        resp.read()
        self.assertEqual(resp.status, 403)
        resp = retry(self.get_notification)
        self.assertIn(resp.status, (200, 204))
        self.assertFalse(resp.text)

        # set read-write acl
        acl_user = tf.swift_test_user[2]
        acl = {'read-write': [acl_user]}
        headers = {'x-account-access-control': json.dumps(acl)}
        resp = retry(self.post_account, headers=headers, use_account=1)
        resp.read()
        self.assertEqual(resp.status, 204)

        # read-write can post notification config
        resp = retry(self.post_notification, self.s3_conf_good, use_account=3)
        resp.read()
        self.assertIn(resp.status, (200, 204))
        resp = retry(self.get_notification)
        self.assertIn(resp.status, (200, 204))
        self.assertTrue(self.s3_conf_good.items() <= resp.json().items())

        # delete notification config
        resp = retry(self.post_notification, use_account=3)
        resp.read()
        self.assertIn(resp.status, (200, 204))
        resp = retry(self.get_notification, use_account=3)
        self.assertIn(resp.status, (200, 204))
        self.assertFalse(resp.text)

        # remove all acl
        headers = {'x-account-access-control': json.dumps({})}
        resp = retry(self.post_account, headers=headers, use_account=1)
        resp.read()
        self.assertEqual(resp.status, 204)

        # no acl => no store
        resp = retry(self.post_notification, self.s3_conf_good, use_account=3)
        resp.read()
        self.assertEqual(resp.status, 403)
        resp = retry(self.get_notification)
        self.assertIn(resp.status, (200, 204))
        self.assertFalse(resp.text)

        # return to initial state
        self.initial_state()


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enoss.filter_rules.irule import IRule
from swift.common.utils import split_path


class PrefixRule(IRule):

    @staticmethod
    def validate(value):
        return type(value) == str

    def __call__(self, app, resp):
        version, account, container, object = split_path(
            resp.environ['PATH_INFO'], 1, 4, rest_with_last=True)
        if object:
            return object.startswith(self.value)
        elif container:
            return container.startswith(self.value)
        return False



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import inspect
import sys


def get_s3_event_name(account, container, object, method):
    res = "s3"
    if object:
        res += ":Object"
    elif container:
        res += ":Bucket"
    else:
        res += ":Account"

    if method in ["PUT", "POST", "COPY"]:
        res += "Created"
    elif method in ["DELETE"]:
        res += "Deleted"
    else:
        res += "Accessed"

    res += ":" + method.title()

    return res


def __get_handler_class_name(handler_name, handler_suffix):
    return handler_name.title() + handler_suffix.title()


def get_destination_handler_name(destination_name):
    return __get_handler_class_name(destination_name, "Destination")


def get_rule_handler_name(rule_name):
    return __get_handler_class_name(rule_name, "Rule")


def get_payload_handler_name(payload_name):
    return __get_handler_class_name(payload_name, "Payload")


def __get_handlers(handler_modules, handler_suffix):
    handlers = {}
    interface_class_name = "I" + handler_suffix.title()
    for handler_module in handler_modules:
        for (handler_name, handler_class) in inspect.getmembers(
                sys.modules[handler_module.__name__], inspect.isclass):
            if handler_name.endswith(handler_suffix) \
                    and handler_name != interface_class_name:
                handlers[handler_name] = handler_class
    return handlers


def get_payload_handlers(payload_modules):
    return __get_handlers(payload_modules, "Payload")


def get_destination_handlers(destination_modules):
    return __get_handlers(destination_modules, "Destination")


def get_rule_handlers(rule_modules):
    return __get_handlers(rule_modules, "Rule")


def _byteify(data, ignore_dicts=False):
    if isinstance(data, str):
        return data
    # if this is a list of values, return list of byteified values
    if isinstance(data, list):
        return [_byteify(item, ignore_dicts=True) for item in data]
    # if this is a dictionary, return dictionary of byteified keys and values
    # but only if we haven't already byteified it
    if isinstance(data, dict) and not ignore_dicts:
        return {
            _byteify(key, ignore_dicts=True): _byteify(val, ignore_dicts=True)
            for key, val in data.items()
        }
    # python 3 compatible duck-typing
    # if this is a unicode string, return its string representation
    if str(type(data)) == "<type 'unicode'>":
        return data.encode('utf-8')
    # if it's anything else, return it in its original form
    return data


json_object_hook = _byteify if sys.version_info[0] < 3 else None



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from unittest.mock import patch

import json
import os

from swift.common.swob import HTTPOk, Request

from swift.common.request_helpers import get_sys_meta_prefix

from enoss.configuration import filter_rule_handlers, \
    ConfigurationInvalid
from enoss.destinations.idestination \
    import IDestination
from enoss.enoss import ENOSSMiddleware
from enoss.filter_rules.irule import IRule
from enoss.payloads.ipayload import IPayload

from test.debug_logger import debug_logger
from test.unit.common.middleware.helpers import FakeSwift


class MockDestination(object):
    def __init__(self, conf):
        self.reset()

    def reset(self):
        self.state = 'notification not sent'

    def send_notification(self, notification):
        self.state = 'notification sent'


class TestENOSS(unittest.TestCase):

    def setUp(self):
        self.fake_swift = FakeSwift()
        self.logger = debug_logger('test-event_notifications')
        self.test_1_init()
        self.s3_notification_conf = {
            "BeanstalkdConfigrations": [
                {
                    "Id": "test",
                    "Events": ["*"],
                    "PayloadStructure": "s3",
                    "Filter": {
                        "Key": {
                            "FilterRules": [{
                                "Name": "suffix",
                                "Value": ".jpg"
                            }]
                        }
                    }
                }
            ]
        }

    def tearDown(self):
        pass

    @classmethod
    def setUpClass(cls):
        with open('/tmp/enoss-destinations.conf', 'w'): pass

    @classmethod
    def tearDownClass(self):
        if os.path.exists('/tmp/enoss-destinations.conf'):
            os.remove('/tmp/enoss-destinations.conf')

    def assertNoRaises(self, test_f, *args, **kwargs):
        try:
            test_f(*args, **kwargs)
        except Exception as e:
            self.fail(str(e))

    @patch('enoss.destinations.BeanstalkdDestination',
           new=MockDestination)
    @patch('enoss.destinations.ElasticsearchDestination',
           new=MockDestination)
    def test_1_init(self):
        app_conf = {
            'use_destinations': 'beanstalkd',
            'destinations_conf_path': '/tmp/enoss-destinations.conf',
            's3_schema': '/etc/swift/enoss/configuration-schema.json'
        }
        # enoss middleware initializes all payload/destination/filter handlers
        self.app = ENOSSMiddleware(self.fake_swift, app_conf,
                                   logger=self.logger)

    def test_2_handlers(self):
        def check_interface(self, module, interface):
            for cls in map(module.__dict__.get, module.__all__):
                self.assertTrue(
                    isinstance(cls, interface) or issubclass(cls, interface)
                )

        import enoss.destinations as dsts
        import enoss.filter_rules as filters
        import enoss.payloads as payloads

        from enoss.destinations.idestination \
            import IDestination as dst_interface
        from enoss.filter_rules.irule \
            import IRule as filter_interface
        from enoss.payloads.ipayload \
            import IPayload as payload_interface


        check_interface(self, dsts, dst_interface)
        check_interface(self, filters, filter_interface)
        check_interface(self, payloads, payload_interface)

    def test_3_configuration(self):
        def validate_config(testing_conf):
            return self.app.configuration_validator.validate(
                self.app.destination_handlers,
                self.app.payload_handlers,
                testing_conf)

        self.assertNoRaises(validate_config, self.s3_notification_conf)

        beanstalkd_conf = self.s3_notification_conf["BeanstalkdConfigrations"]

        # unsupported configuration name
        self.s3_notification_conf["InvalidNameConfiguration"] = beanstalkd_conf
        self.assertRaises(ConfigurationInvalid, validate_config, \
            self.s3_notification_conf)
        del self.s3_notification_conf["InvalidNameConfiguration"]

        # unsupported event type
        beanstalkd_conf[0]["Events"].append("BadEventType")
        self.assertRaises(ConfigurationInvalid, validate_config, \
            self.s3_notification_conf)
        del beanstalkd_conf[0]["Events"][-1]

        # missing value for filter operator
        conf_rules = beanstalkd_conf[0]["Filter"]["Key"]["FilterRules"]
        conf_rules.append({"Name": "suffix"})
        self.assertRaises(ConfigurationInvalid, validate_config, \
            self.s3_notification_conf)

        # valid filter operator with value
        conf_rules[-1] = {"Name": "suffix", "Value": "a"}
        self.assertNoRaises(validate_config, self.s3_notification_conf)

        # non existing filter operator
        conf_rules[-1] = {"Name": "non_existing", "Value": "1"}
        conf_json = json.dumps(self.s3_notification_conf)
        self.assertRaises(ConfigurationInvalid, validate_config, \
            self.s3_notification_conf)
        del conf_rules[-1]

        # not supported payload structure
        beanstalkd_conf[0]["PayloadStructure"] = "NotSupportedPayloadStructure"
        conf_json = json.dumps(self.s3_notification_conf)
        self.assertRaises(ConfigurationInvalid, validate_config, \
            self.s3_notification_conf)

        # default payload structure is s3 which is supported
        del beanstalkd_conf[0]["PayloadStructure"]
        conf_json = json.dumps(self.s3_notification_conf)
        self.assertNoRaises(ConfigurationInvalid, validate_config, \
            self.s3_notification_conf)

        # one destination can have multiple configurations
        beanstalkd_conf.append(beanstalkd_conf[0])
        conf_json = json.dumps(self.s3_notification_conf)
        self.assertNoRaises(ConfigurationInvalid, validate_config, \
            self.s3_notification_conf)

    def test_4_post_configuration_valid(self):
        self.test_1_init()
        self.fake_swift.register('POST', '/v1/a/c1', HTTPOk, {}, 'passed')
        beanstalkd = self.app.destination_handlers["BeanstalkdDestination"]
        beanstalkd.reset()

        infocache = {
            'account/a': {
                'meta': {}
            },
            'container/a/c1': {
                'sysmeta': {
                    'notifications': json.dumps(self.s3_notification_conf)
                }
            }
        }

        req = Request.blank(
            '/v1/a/c1?notification',
            environ={'REQUEST_METHOD': 'POST', 'swift.infocache': infocache},
            body=json.dumps(self.s3_notification_conf)
        )
        res = req.get_response(self.app)
        self.assertEqual(res.status_int, 200)
        # check if test event is sent
        self.assertEqual(beanstalkd.state, 'notification sent')

        stored_configuration = req.headers.get(get_sys_meta_prefix("container")
                                               + "notifications")
        # check if configuration is stored to sysmetadata
        self.assertIsInstance(stored_configuration, str)
        stored_configuration = json.loads(stored_configuration)
        for key in self.s3_notification_conf:
            self.assertEqual(self.s3_notification_conf[key],
                             stored_configuration[key])

    def test_5_post_configuration_invalid(self):
        self.test_1_init()
        beanstalkd = self.app.destination_handlers["BeanstalkdDestination"]
        beanstalkd.reset()
        self.fake_swift.register('POST', '/v1/a/c1', HTTPOk, {}, 'passed')
        self.fake_swift.register('POST', '/v1/a/c1/o1', HTTPOk, {}, 'passed')

        infocache = {'account/a': {'meta': {}}, 'container/a/c1': {'meta': {}}}

        invalid_notification_configuration = {"invalid": True}
        req = Request.blank(
            '/v1/a/c1?notification',
            environ={'REQUEST_METHOD': 'POST', 'swift.infocache': infocache},
            body=json.dumps(invalid_notification_configuration)
        )
        res = req.get_response(self.app)
        self.assertEqual(res.status_int, 400)

        # notification configuration can't be stored in object level
        req = Request.blank(
            '/v1/a/c1/o1?notification',
            environ={'REQUEST_METHOD': 'POST', 'swift.infocache': infocache},
            body=json.dumps(self.s3_notification_conf)
        )
        res = req.get_response(self.app)
        self.assertEqual(res.status_int, 403)
        self.assertEqual(beanstalkd.state, 'notification not sent')

    def test_6_read_configuration(self):
        self.fake_swift.register('GET', '/v1/a2', HTTPOk, {}, 'passed')
        self.fake_swift.register('GET', '/v1/a2/c2', HTTPOk, {}, 'passed')

        infocache = {'account/a2': {'sysmeta': {}},
                     'container/a2/c2': {'sysmeta': {}}}
        req = Request.blank(
            '/v1/a2/c2?notification',
            environ={'REQUEST_METHOD': 'GET', 'swift.infocache': infocache}
        )
        res = req.get_response(self.app)
        self.assertEqual(res.status_int, 200)
        self.assertEqual(res.body, b'')

        req = Request.blank(
            '/v1/a2?notification',
            environ={'REQUEST_METHOD': 'GET', 'swift.infocache': infocache}
        )
        res = req.get_response(self.app)
        self.assertEqual(res.status_int, 200)
        self.assertEqual(res.body, b'')

        infocache = {
            'account/a2': {'sysmeta': {}},
            'container/a2/c2': {
                'sysmeta': {
                    'notifications': json.dumps(self.s3_notification_conf)
                }
            }
        }
        req = Request.blank(
            '/v1/a2/c2?notification',
            environ={'REQUEST_METHOD': 'GET', 'swift.infocache': infocache}
        )
        res = req.get_response(self.app)
        self.assertEqual(res.status_int, 200)
        body = res.body.decode('utf8').replace("'", '"')
        self.assertEqual(json.loads(body), self.s3_notification_conf)

        infocache = {
            'account/a2': {
                'sysmeta': {
                    'notifications': json.dumps(self.s3_notification_conf)
                }
            },
            'container/a2/c2': {'sysmeta': {}}
        }
        req = Request.blank(
            '/v1/a2?notification',
            environ={'REQUEST_METHOD': 'GET', 'swift.infocache': infocache}
        )
        res = req.get_response(self.app)
        self.assertEqual(res.status_int, 200)
        body = res.body.decode('utf8').replace("'", '"')
        self.assertEqual(json.loads(body), self.s3_notification_conf)

    def test_7_send_notification_container_level(self):
        self.test_1_init()
        self.fake_swift.register('GET', '/v1/a3/c3/o3',
                                 HTTPOk, {}, 'passed')
        self.fake_swift.register('GET', '/v1/a3/c3/o3.jpg',
                                 HTTPOk, {}, 'passed')

        beanstalkd = self.app.destination_handlers["BeanstalkdDestination"]
        beanstalkd.reset()

        infocache = {
            'account/a3': {'sysmeta': {}},
            'container/a3/c3': {
                'sysmeta': {
                    'notifications': json.dumps(self.s3_notification_conf)
                }
            }
        }
        req = Request.blank(
            '/v1/a3/c3/o3',
            environ={'REQUEST_METHOD': 'GET', 'swift.infocache': infocache}
        )
        res = req.get_response(self.app)
        self.assertEqual(res.status_int, 200)
        self.assertEqual(beanstalkd.state, 'notification not sent')

        beanstalkd.reset()
        req = Request.blank(
            '/v1/a3/c3/o3.jpg',
            environ={'REQUEST_METHOD': 'GET', 'swift.infocache': infocache}
        )
        res = req.get_response(self.app)
        self.assertEqual(res.status_int, 200)
        self.assertEqual(beanstalkd.state, 'notification sent')

    def test_8_send_notification_account_level(self):
        self.test_1_init()
        self.fake_swift.register('GET', '/v1/a4/c4', HTTPOk, {}, 'passed')
        self.fake_swift.register('GET', '/v1/a4/c4.jpg', HTTPOk, {}, 'passed')

        beanstalkd = self.app.destination_handlers["BeanstalkdDestination"]
        beanstalkd.reset()

        infocache = {
            'account/a4': {
                'sysmeta': {
                    'notifications': json.dumps(self.s3_notification_conf)
                }
            },
            'container/a4/c4': {'sysmeta': {}}
        }
        req = Request.blank(
            '/v1/a4/c4',
            environ={'REQUEST_METHOD': 'GET', 'swift.infocache': infocache}
        )
        res = req.get_response(self.app)
        self.assertEqual(res.status_int, 200)
        self.assertEqual(beanstalkd.state, 'notification not sent')

        beanstalkd.reset()
        req = Request.blank(
            '/v1/a4/c4.jpg',
            environ={'REQUEST_METHOD': 'GET', 'swift.infocache': infocache}
        )
        res = req.get_response(self.app)
        self.assertEqual(res.status_int, 200)
        self.assertEqual(beanstalkd.state, 'notification sent')
        # todo fix ResourceWarning


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enoss.destinations.idestination import IDestination

import json
import sys

IS_PY2 = sys.version_info[0] < 3

if IS_PY2:
    from beanstalk.serverconn import ServerConn
else:
    from greenstalk import Client

class BeanstalkdDestination(IDestination):
    def __init__(self, conf):
        self.conf = conf["beanstalkd"]
        self.tube = self.conf.get("tube", "default")
        if IS_PY2:
          self.connection = ServerConn(
            self.conf["addr"], int(self.conf["port"]))
        else:
            self.connection = Client(
              (self.conf["addr"], int(self.conf["port"])))
        self.connection.use(self.tube)

    def __del__(self):
        if self.connection:
            self.connection.close()

    def send_notification(self, notification):
        self.connection.put(json.dumps(notification))



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enoss.filter_rules.irule import IRule


class HttpcodesRule(IRule):

    @staticmethod
    def validate(values):
        # values must be a list of strings (e.g. ["200", "404", "4xx"])
        allowed_chars = ['x', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9']
        if type(values) != list:
            return False
        for val in values:
            if type(val) != str or any(x not in allowed_chars for x in val):
                return False
        return True

    def _cmp(self, val_1, val_2):
        if len(val_1) != len(val_2):
            return False
        for i in range(len(val_1)):
            if val_1[i] != val_2[i] and val_2[i] != 'x':
                return False
        return True

    def __call__(self, app, resp):
        status = str(resp.status_int)
        return any(self._cmp(status, x) for x in self.value)



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enoss.filter_rules.irule import IRule


class UsersinRule(IRule):

    @staticmethod
    def validate(values):
        # list of strings
        return type(values) == list and all(type(x) == str for x in values)

    def __call__(self, app, resp):
        user = resp.environ.get("REMOTE_USER")
        return user in self.values


class UsersoutRule(IRule):

    @staticmethod
    def validate(values):
        # list of strings
        return type(values) == list and all(type(x) == str for x in values)

    def __call__(self, app, resp):
        user = resp.environ.get("REMOTE_USER")
        return user not in self.values



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_namespace_packages

VERSION = '0.0.1-3'
DESCRIPTION = 'Event Notifications in OpenStack Swift'

with open("README.rst", "r", encoding="utf-8") as fh:
    LONG_DESCRIPTION = fh.read()

# Setting up
setup(
       # the name must match the folder name 'verysimplemodule'
        name="enoss",
        version=VERSION,
        author="Nemanja Vasiljevic",
        author_email="<xvasil03@gmail.com>",
        url="https://github.com/xvasil03/enoss",
        license="Apache Software",
        zip_safe=False,
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        long_description_content_type="text/x-rst",
        packages=find_namespace_packages(include=["enoss*"]),
        install_requires=["jsonschema"],
        keywords=['openstack swift', 'event notifications'],
        classifiers=[
            "Development Status :: 5 - Production/Stable",
            "Environment :: OpenStack",
            "Intended Audience :: Information Technology",
            "Intended Audience :: System Administrators",
            "License :: OSI Approved :: Apache Software License",
            "Operating System :: POSIX :: Linux",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9"
        ],
        options={"bdist_wheel": {"universal": "1"}}
)



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enoss.filter_rules.contenttype import ContenttypeRule
from enoss.filter_rules.httpcodes import HttpcodesRule
from enoss.filter_rules.prefix import PrefixRule
from enoss.filter_rules.suffix import SuffixRule
from enoss.filter_rules.users import UsersinRule, UsersoutRule
from enoss.filter_rules.size import MinsizeRule, MaxsizeRule

__all__ = [
    'PrefixRule',
    'SuffixRule',
    'HttpcodesRule',
    'UsersinRule',
    'UsersoutRule',
    'MinsizeRule',
    'MaxsizeRule',
    'ContenttypeRule'
]



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enoss.destinations.idestination import IDestination

from elasticsearch import Elasticsearch
import json


class ElasticsearchDestination(IDestination):
    def __init__(self, conf):
        self.conf = conf["elasticsearch"]
        self.target_index = self.conf["index"]
        self.es = Elasticsearch(
            self.conf["hosts"],
            ca_certs=self.conf["ca_certs"],
            basic_auth=(self.conf["auth_user"], self.conf["auth_passwd"])
        )
        assert (self.es.ping()), "Cannot connect to elasticsearch"
        # create index if doesnt exist
        if not self.es.indices.exists(index=self.target_index):
            index_mappings = self._get_mappings(
                self.conf.get("index_mappings_file")
            )
            self.es.indices.create(
                index=self.target_index,
                mappings=index_mappings)

    def _get_mappings(self, file_path):
        mapping = None
        if file_path:
            with open(file_path) as f:
                mapping = json.loads(f.read())
        return mapping

    def send_notification(self, notification):
        self.es.index(
            index=self.target_index,
            body=json.dumps(notification)
        )



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

from enoss.destinations.idestination import IDestination

import json
from kafka import KafkaProducer


class KafkaDestination(IDestination):
    def __init__(self, conf):
        self.conf = conf["kafka"]
        conn_conf = self._get_conn_conf()
        self.conn = KafkaProducer(**conn_conf)
        self.topic = self.conf["topic"]

    def _get_conn_conf(self):
        conn_prefix = "conn_"
        conn_conf = {key[len(conn_prefix):]:value \
            for key, value in self.conf.items() \
            if key.startswith(conn_prefix)}
        return conn_conf

    def __del__(self):
        self.conn.flush()
        self.conn.close()

    def send_notification(self, notification):
        self.conn.send(self.topic, json.dumps(notification).encode())



# Copyright (c) 2022 Nemanja Vasiljevic <xvasil03@gmail.com>.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import json
import jsonschema

from swift.common.utils import split_path
from enoss.utils import (
    get_s3_event_name, get_rule_handlers, get_rule_handler_name,
    get_destination_handler_name, get_payload_handler_name, json_object_hook)
from enoss.constants import supported_s3_events
import enoss.filter_rules as filter_rules_module

filter_rule_handlers = get_rule_handlers([filter_rules_module])


def _remove_suffix(input, suffix):
    return input[:-len(suffix)]

class ConfigurationInvalid(Exception):
    pass


class S3ConfigurationValidator(object):
    def __init__(self, schema_path):
        with open(schema_path, 'r') as file:
            self.schema = json.load(file, object_hook=json_object_hook)

    def validate_schema(self, config_json):
        try:
            jsonschema.validate(instance=config_json, schema=self.schema)
        except jsonschema.ValidationError:
            raise ConfigurationInvalid("Invalid configuration")
        return config_json

    def validate_event_type(self, config):
        for _, destinations_configuration in config.items():
            for event_configuration in destinations_configuration:
                if any(event not in supported_s3_events
                       for event in event_configuration["Events"]):
                    raise ConfigurationInvalid("Unsupported event type")

    def validate_rules(self, config):
        for _, destinations_configuration in config.items():
            for event_configuration in destinations_configuration:
                filter_conf = event_configuration.get("Filter", {})
                for _, filter_item in filter_conf.items():
                    for filter_rule in filter_item["FilterRules"]:
                        handler_name = get_rule_handler_name(
                            filter_rule["Name"])
                        handler = filter_rule_handlers.get(handler_name)
                        err_msg = None
                        if not handler:
                            err_msg = "Unsupported rule operator"
                        elif not handler.validate(filter_rule["Value"]):
                            err_msg = "Invalid rule value"
                        if err_msg:
                            raise ConfigurationInvalid(err_msg)

    def validate_destinations(self, destination_handlers, config):
        for destination_configs in config:
            destination_name = _remove_suffix(
                destination_configs, "Configrations")
            handler_name = get_destination_handler_name(destination_name)
            if handler_name not in destination_handlers:
                raise ConfigurationInvalid("Unsupported destination")

    def validate_payload_structure(self, payload_handlers, config):
        for _, destinations_configuration in config.items():
            for notification_conf in destinations_configuration:
                payload_name = notification_conf.get("PayloadStructure", "s3")
                handler_name = get_payload_handler_name(payload_name)
                if handler_name not in payload_handlers:
                    raise ConfigurationInvalid("Unsupported payload structure")

    def validate(self, destination_handlers, payload_handlers, config_json):
        self.validate_schema(config_json)
        self.validate_event_type(config_json)
        self.validate_rules(config_json)
        self.validate_destinations(destination_handlers, config_json)
        self.validate_payload_structure(payload_handlers, config_json)


class S3NotifiationConfiguration(object):

    class DestinationConfiguration(object):

        class FilterConfiguration(object):
            def __init__(self, key, config):
                self.key = key
                self.config = config
                self.rules = []
                for rule in config["FilterRules"]:
                    rule_handler_name = get_rule_handler_name(rule["Name"])
                    rule_handler = filter_rule_handlers[rule_handler_name]
                    self.rules.append(rule_handler(rule["Value"]))

            def does_satisfy(self, app, resp):
                return all(rule(app, resp) for rule in self.rules)

        def __init__(self, config):
            self.config = config
            self.id = config["Id"]
            self.allowed_events = config["Events"]
            self.payload_type = config.get("PayloadStructure", "s3")
            self.only_succ_events = config.get("OnlySuccessfulEvents", True)
            filer_configs = config.get("Filter", {})
            self.filters = [self.FilterConfiguration(filter_key, filter_config)
                            for filter_key, filter_config
                            in filer_configs.items()]

        def is_allowed_event(self, resp):
            version, account, container, object = split_path(
                resp.environ['PATH_INFO'], 1, 4, rest_with_last=True)
            method = resp.environ.get('swift.orig_req_method',
                                      resp.request.method)
            event = get_s3_event_name(account, container, object, method)
            for allowed_event in self.allowed_events:
                if allowed_event.endswith("*"):
                    if event.startswith(allowed_event[:-1]):
                        return True
                else:
                    if allowed_event == event:
                        return True
            return False

        def is_satisfied_rule(self, app, resp):
            return not self.filters or any(filter.does_satisfy(app, resp)
                                           for filter in self.filters)

        def does_satisfy(self, app, resp):
            return self.is_allowed_event(resp) \
                and self.is_satisfied_rule(app, resp) \
                and (resp.is_success or not self.only_succ_events)

    def __init__(self, config):
        self.config = config if type(config) == dict \
            else json.loads(config, object_hook=json_object_hook)
        self.destinations_configurations = {}
        for dest_confs_name, dest_confs in self.config.items():
            for dest_conf in dest_confs:
                # <dest_name>Configrations => <dest_name>
                dest_name = _remove_suffix(
                    dest_confs_name, "Configrations").lower()
                new_dest_conf = self.DestinationConfiguration(dest_conf)
                self.destinations_configurations.setdefault(dest_name, [])\
                                                .append(new_dest_conf)

    def get_satisfied_destinations(self, app, resp):
        result = {}
        for dest_name, dest_confs in self.destinations_configurations.items():
            for dest_conf in dest_confs:
                if dest_conf.does_satisfy(app, resp):
                    result.setdefault(dest_name, []).append(dest_conf)
        return result
